import pandas as pd
import os
from pandas.util.testing import assert_frame_equal

from ovretl.tracking_utils.determine_active_shipments import determine_active_shipments

cwd = os.path.dirname(__file__)


def test_determine_active_shipments():
    raw_df = pd.read_csv(os.path.join(cwd, "shipments_clean.csv"), parse_dates=True)
    raw_df.loc[:, "delivery_date"] = pd.to_datetime(raw_df["delivery_date"])
    raw_df.loc[:, "arrival_date"] = pd.to_datetime(raw_df["arrival_date"])
    result = determine_active_shipments(raw_df, "2020-09-17")
    result_should_be = pd.read_csv(os.path.join(cwd, "active_shipments.csv"), parse_dates=True)
    assert_frame_equal(
        result.reset_index(drop=True),
        result_should_be.reset_index(drop=True),
        check_dtype=False,
        check_column_type=False,
        check_less_precise=True,
        check_names=False,
        check_exact=False,
        check_datetimelike_compat=True,
    )



import pandas as pd


def add_prices_ids_to_shipments(shipments_df: pd.DataFrame, final_checks_df: pd.DataFrame, billings_df: pd.DataFrame):
    shipments_with_ids = shipments_df[["shipment_id", "proposition_id", "foresea_name"]]
    shipments_with_ids = pd.merge(
        shipments_with_ids,
        final_checks_df[["shipment_id", "id", "initial_proposition_id"]],
        on="shipment_id",
        how="left",
    ).rename(columns={"id": "final_check_id"})

    shipments_with_ids = pd.merge(
        shipments_with_ids, billings_df[["shipment_id", "id"]], on="shipment_id", how="left",
    ).rename(columns={"id": "billing_id"})

    return shipments_with_ids[
        ~shipments_with_ids.duplicated(subset=["final_check_id"]) | shipments_with_ids["final_check_id"].isnull()
    ]



import pandas as pd


def get_total_workflows_weights(
    so_workflows: pd.DataFrame,
    so_states: pd.DataFrame,
    so_state_action_associations: pd.DataFrame,
    so_actions: pd.DataFrame,
) -> pd.DataFrame:
    so_states = so_states.rename(columns={"id": "so_state_id"})
    so_actions = so_actions.rename(columns={"id": "so_action_id"})
    so_state_action_associations = so_state_action_associations.rename(columns={"id": "so_state_action_association_id"})
    so_workflows = pd.merge(so_workflows, so_states, how="left", left_on="id", right_on="so_workflow_id")
    so_workflows = pd.merge(so_workflows, so_state_action_associations, how="left", on="so_state_id")
    so_workflows = pd.merge(so_workflows, so_actions, how="left", on="so_action_id")
    so_workflows = so_workflows.loc[:, ["id", "weight"]].groupby(by="id").sum()
    return so_workflows


def so_workflows_weights(
    so_workflows: pd.DataFrame,
    so_states: pd.DataFrame,
    so_state_action_associations: pd.DataFrame,
    so_actions: pd.DataFrame,
) -> pd.DataFrame:
    so_workflows_weight = get_total_workflows_weights(
        so_workflows=so_workflows,
        so_states=so_states,
        so_state_action_associations=so_state_action_associations,
        so_actions=so_actions,
    )
    so_workflows = pd.merge(so_workflows, so_workflows_weight, on="id", how="left")
    so_workflows_weight = so_workflows.rename(columns={"weight": "total_workflow_weight"})
    return so_workflows_weight.loc[:, ["id", "name", "total_workflow_weight"]]



import psycopg2
import pandas as pd
from ovretl.db_utils.fetch_db_credentials import fetch_db_credentials


def execute_simple_insert(df: pd.DataFrame, table_name: str, dbname: str, stage="dev"):
    values_to_insert = [tuple(x) for x in df.values]
    columns = '"{}"'.format('","'.join(df.columns))
    credentials = fetch_db_credentials(stage)
    conn = psycopg2.connect(
        host=credentials["host"],
        user=credentials["username"],
        password=credentials["password"],
        dbname=dbname,
        port=6543,
    )
    cur = conn.cursor()
    from psycopg2.extras import execute_values

    execute_values(
        cur,
        """INSERT INTO "{}" (
                  {}
                  ) VALUES %s""".format(
            table_name, columns
        ),
        values_to_insert,
    )
    cur.close()
    conn.commit()
    conn.close()
    print("Inserted {} rows.".format(cur.rowcount))
